from .artifact_store import AzureDataLakeClient
from . import orchestrator
import os
import sys

class Orca:
    def __init__(self):
        pass
    
    def create_job(
            self,
            job_name : str,
            file_name : str,
            file_path : str,
            run_type : str,
            version_id : int,
            max_concurrent_runs : int,
            timeout_seconds : int,
            schedule : str,
            airflow_config : str = '',
            config : bool = True,
            cluster_id : str = None,
            storage_account_name : str = None,
            storage_account_key : str = None,
            host : str= None,
            headers : dict = None,
            filesystem_name : str = None,
            user_name : str = None,
            email_notifications : dict = None,
            repo_name : str = None,
            project : str = None,
            write_out_to_config : bool = False,
            timezone_id : str = "America/New_York",
            pause_status : str = "PAUSED",
    ):
        """

        This function is the main work horse for the entire tool.
        You can use it to create jobs on databricks, which can then be scheduled and run
        through airflow or any other orchestration engine.
        
        Notes:
        Use https://www.freeformatter.com/cron-expression-generator-quartz.html 
        for the schedule formatting.

        Use https://garygregory.wordpress.com/2013/06/18/what-are-the-java-timezone-ids/
        for the timezone id.

        Parameters
        ----------
        * job_name : str - the name of the job
        
        * file_name : str - the name of the file to upload
        
        * file_path : str - the path without the file_name. 

        Default : ./ 

        Note: The author typically only will upload from 
        the current working directory unless I'm traversing 
        a set of files and want that structure maintained
        on the databricks mount.
        
        * run_type : str - this is an enumeration, technically.
        You can set it to be whatever you want, but the convention
        the author likes is:
        * test
        * debug
        * production
        
        * version_id : int - the version of the code to upload,
        helpful for testing and debugging.
        
        * max_concurrent_runs : int - the maximum concurrent runs
        that can be performed.
        
        * timeout_seconds : int - the number of seconds to allow
        before timing out if the job isn't doing any work actively.
        The author typically sets this to 30, but use your own best 
        judgement.
        
        Default : 30
        
        * schedule : str - this is the schedule the job should run on
        so like every minute, every day, every day at noon.  The
        syntax for the scheduler is confusing, but this formatter
        helps:
        
        https://www.freeformatter.com/cron-expression-generator-quartz.html 
        
        * airflow_config [optional] : str - a string containing the job id's and 
        task paths.  If write_out_to_config=False, it is recommended to make use of
        this parameter.  

        * config [optional] : bool - whether to read from a config file, named
        config.py or not.
        Note: must be in the same directory as the tool is being run from.
        
        The following fields are optional only if they exist in the config.
        Otherwise they are required:
        
        * cluster_id [optional] : str - an existing cluster id to run the job on.
        
        * storage_account_name [optional] : str - an azure datalake storage account name.

        * storage_account_key [optional] : str - an azure datalake storage account key.
        
        Note: the storage account MUST be mounted in databricks using the mount function below,
        or the jobs will not be available in databricks.  The mount function is called:
        `databricks_mount_container_blob`
        
        * host [optional] : str - the host IP Address
    
        * headers [optional] : dict - the headers come from the config 
        and should not be changed  

        * filesystem_name [optional] : str - the name of the azure datalake filesystem.
        
        * user_name [optional] : str - the email address associated with a databricks account.
        
        * email_notifications [optional] : bool - whether to notify the user by email on
        job failures.

        * repo_name [optional] : str - the name of the github repo 
        (or other version control system repo name).

        * project [optional] : str - the name of the project.
        
        END of 'semi optional' parameters
        
        * write_out_to_config [optional] : bool - if the parameters 
        were passed in to the function, if set to True, then they will
        be written out to a file called config.py
        
        * timezone_id [optional] : str - this is optional in the sense
        that a default parameter value is passed.  It is _required_.
        
        Default: "America/New_York"
        
        For more options see: 

        https://garygregory.wordpress.com/2013/06/18/what-are-the-java-timezone-ids/

        * pause_status : str - possible values:
        
        * "PAUSED"
        * "UNPAUSED"

        If job is paused, then it doesn't run even when executed.  To 'actually' run
        it must be in the UNPAUSED state.
        
        """
        self.max_concurrent_runs = max_concurrent_runs
        self.job_name = job_name
        self.file_name = file_name
        self.file_path = file_path
        self.run_type = run_type
        self.version_id = version_id
        self.cluster_id = cluster_id
        self.storage_account_name = storage_account_name
        self.storage_account_key = storage_account_key
        self.host = host
        self.headers = headers
        self.filesystem_name = filesystem_name
        self.user_name = user_name
        self.email_notifications = email_notifications
        self.timeout_seconds = timeout_seconds
        self.repo_name = repo_name
        self.project = project
        if config:
            self.read_config()

        task_file_path = self.upload_task(
            self.file_name,
            storage_account_name=self.storage_account_name,
            storage_account_key=self.storage_account_key,
            run_type=self.run_type,
            version_id=self.version_id,
            repo_name=self.repo_name,
            file_path=self.file_path,
            filesystem_name=self.filesystem_name,
            project=self.project,
            config=False
        )

        job_format = "SINGLE_TASK"
        tasks = [
        {
            "task_key": self.job_name,
            "description": "",
            "depends_on": [],
            "existing_cluster_id": self.cluster_id,
            "spark_python_task": {
                "python_file": task_file_path,
                "parameters": []
            },
            "timeout_seconds": 1000,
            "max_retries": 3,
            "min_retry_interval_millis": 10000,
            "retry_on_timeout": "true",
        }]
        job_clusters = []
        schedule = {
            "quartz_cron_expression": f"{schedule}",
            "timezone_id": timezone_id,
            "pause_status": pause_status
        }
        max_concurrent_runs = self.max_concurrent_runs

        access_control_list = [
            {"user_name": self.user_name,
             "permission_level": "IS_OWNER"}
        ]
        job = orchestrator.create_job(
            self.job_name,
            tasks,
            job_clusters,
            self.email_notifications,
            timeout_seconds,
            schedule,
            self.max_concurrent_runs,
            job_format,
            access_control_list,
            self.host,
            self.headers
        )
        
        print(f"Task File Path: {task_file_path}")
        print(f"Job ID: {int(job.json()['job_id'])}")

        if write_out_to_config:
            job_identifier = file_name.split(".")[0]
            with open(f"airflow_config_{self.repo_name}_{self.run_type}_{self.version_id}.py", "a") as f:
                f.write(f"job_{job_identifier} = {int(job.json()['job_id'])}\n")
                f.write(f"task_{job_identifier} = '{task_file_path}'\n")
        else:    
            airflow_config += f"job_{job_identifier} = {int(job.json()['job_id'])}\n"
            airflow_config += f"task_{job_identifier} = '{task_file_path}'\n"
        return airflow_config
            
    
    def upload_task(
            self,
            file_name : str,
            run_type : str,
            storage_account_name : str = None,
            storage_account_key : str = None,
            version_id : int = None,
            repo_name : str = None,
            file_path : str = None,
            filesystem_name : str = None,
            project : str = None,
            config : bool = True,
            mode : str = 'r'
    ):
        """
        Uploads a task to azure datalake.
        
        Parameters
        -----------
        
        * file_name : str - the name of the file to upload
                
        * file_path : str - the path without the file_name. 

        Default : ./ 

        Note: The author typically only will upload from 
        the current working directory unless I'm traversing 
        a set of files and want that structure maintained
        on the databricks mount.

        * run_type : str - this is an enumeration, technically.
        You can set it to be whatever you want, but the convention
        the author likes is:
        * test
        * debug
        * production
        
        * version_id : int - the version of the code to upload,
        helpful for testing and debugging.
        
        * storage_account_name [optional] : str - an azure datalake storage account name.

        * storage_account_key [optional] : str - an azure datalake storage account key.
        
        Note: the storage account MUST be mounted in databricks using the mount function below,
        or the jobs will not be available in databricks.  The mount function is called:
        `databricks_mount_container_blob`
        
        * filesystem_name [optional] : str - the name of the azure datalake filesystem.
        
        * repo_name [optional] : str - the name of the github repo 
        (or other version control system repo name).

        * project [optional] : str - the name of the project.

        * config [optional] : bool - if True read from the config.py
        Don't read if False.
        """
        if config:
            self.read_config()

            storage_account_name = self.storage_account_name
            storage_account_key = self.storage_account_key
            repo_name = self.repo_name
            filesystem_name = self.filesystem_name
            project = self.project

        datalake_client = AzureDataLakeClient(
            storage_account_name,
            filesystem_name,
            storage_account_key=storage_account_key
        )
        # add directory variable pass through
        if file_path == './':
            directory = f"{repo_name}/{run_type}/{version_id}/"
        else:
            directory = f"{repo_name}/{run_type}/{version_id}/{file_path}"
        datalake_client.create_directory(directory)
        datalake_client.upload_file(file_path+file_name, directory+file_name, mode=mode)
        return f"dbfs:/mnt/{project}/{directory}{file_name}"


    def execute_job(self, job_id):
        """
        Executes a give job.
        
        Parameters
        ----------
        
        * job_id : int - the job id to execute
        on databricks.
        """
        orchestrator.execute_job(
            job_id, 
            host,
            headers,
            jar_params,
            notebook_params,
            python_params,
            spark_submit_params
        )
        
    def read_config(self):
        """
        Reads a config.py in the current directory.
        """
        sys.path.append(os.getcwd())
        from config import (
            headers, host,
            storage_account_name,
            storage_account_key,
            cluster_id, filesystem_name,
            user_name, email_notifications,
            repo_name, project
        )
        self.headers = headers
        self.host = host
        self.storage_account_name = storage_account_name
        self.storage_account_key = storage_account_key
        self.cluster_id = cluster_id
        self.filesystem_name = filesystem_name
        self.user_name = user_name
        self.email_notifications = email_notifications
        self.repo_name = repo_name
        self.project = project

def create_jobs_and_upload(
        file_names, job_names, run_type, version_id,
        max_concurrent_runs, timeout_seconds, schedule,
        config=True, cluster_id=None, storage_account_name=None,
        storage_account_key=None, host=None, headers=None, filesystem_name=None,
        user_name=None, email_notifications=None, repo_name=None, project=None,
        write_out_to_config=False, airflow_config=''):
    """
    Creates a set of jobs.
    
    Parameters
    ----------
    * job_names : list - the names of the jobs

    * file_names : list - the names of the files to upload

    * run_type : str - this is an enumeration, technically.
    You can set it to be whatever you want, but the convention
    the author likes is:
    * test
    * debug
    * production

    * version_id : int - the version of the code to upload,
    helpful for testing and debugging.

    * max_concurrent_runs : int - the maximum concurrent runs
    that can be performed.

    * timeout_seconds : int - the number of seconds to allow
    before timing out if the job isn't doing any work actively.
    The author typically sets this to 30, but use your own best 
    judgement.

    Default : 30

    * schedule : str - this is the schedule the job should run on
    so like every minute, every day, every day at noon.  The
    syntax for the scheduler is confusing, but this formatter
    helps:

    https://www.freeformatter.com/cron-expression-generator-quartz.html 

    * airflow_config [optional] : str - a string containing the job id's and 
    task paths.  If write_out_to_config=False, it is recommended to make use of
    this parameter.  

    * config [optional] : bool - whether to read from a config file, named
    config.py or not.
    Note: must be in the same directory as the tool is being run from.

    The following fields are optional only if they exist in the config.
    Otherwise they are required:

    * cluster_id [optional] : str - an existing cluster id to run the job on.

    * storage_account_name [optional] : str - an azure datalake storage account name.

    * storage_account_key [optional] : str - an azure datalake storage account key.

    Note: the storage account MUST be mounted in databricks using the mount function below,
    or the jobs will not be available in databricks.  The mount function is called:
    `databricks_mount_container_blob`

    * host [optional] : str - the host IP Address

    * headers [optional] : dict - the headers come from the config 
    and should not be changed  

    * filesystem_name [optional] : str - the name of the azure datalake filesystem.

    * user_name [optional] : str - the email address associated with a databricks account.

    * email_notifications [optional] : bool - whether to notify the user by email on
    job failures.

    * repo_name [optional] : str - the name of the github repo 
    (or other version control system repo name).

    * project [optional] : str - the name of the project.

    END of 'semi optional' parameters

    * write_out_to_config [optional] : bool - if the parameters 
    were passed in to the function, if set to True, then they will
    be written out to a file called config.py

    * timezone_id [optional] : str - this is optional in the sense
    that a default parameter value is passed.  It is _required_.

    Default: "America/New_York"

    For more options see: 

    https://garygregory.wordpress.com/2013/06/18/what-are-the-java-timezone-ids/

    * pause_status : str - possible values:

    * "PAUSED"
    * "UNPAUSED"

    If job is paused, then it doesn't run even when executed.  To 'actually' run
    it must be in the UNPAUSED state.
    """
    
    orca = Orca()
    for index, file_name in enumerate(file_names):
        # we set a convention that the filepath
        # is always the root folder, for now
        # if this becomes unweildy we can change it later
        file_path = "./"
        job_name = job_names[index]
        airflow_config += orca.create_job(
            job_name, file_name, file_path, run_type,
            version_id, max_concurrent_runs, timeout_seconds,
            schedule, config=config, cluster_id=cluster_id,
            storage_account_name=storage_account_name,
            storage_account_key=storage_account_key, host=host,
            headers=headers, filesystem_name=filesystem_name,
            user_name=user_name, email_notifications=email_notifications,
            repo_name=repo_name, project=project,
            write_out_to_config=write_out_to_config,
            airflow_config=airflow_config
        )
    return airflow_config

def databricks_mount_container_blob(
  storage_account_name: str,
  container_name: str,
  secrets_scope: str,
  secrets_key: str
):
  '''
  Mounts a datalake file system to databricks.
  
  Parameters
  ----------
  storage_account_name - name of azure storage account
  container_name - name of container in azure storage account
  secrets_scope - databricks secrets scope name
  secrets_key - databricks secrets key in secrets scope
  '''
  access_key = dbutils.secrets.get(scope=secrets_scope, key=secrets_key)
  conf_key = f"fs.azure.account.key.{storage_account_name}.blob.core.windows.net"
  
  try:
    dbutils.fs.mount(
      source = f"wasbs://{container_name}@{storage_account_name}.blob.core.windows.net",
      mount_point = f"/mnt/{container_name}",
      extra_configs = {conf_key: access_key})
    
    print(f"{container_name} container from {storage_account_name} storage account has been mounted with mountname {container_name}")
    
  except Exception as e:
    print(f"Error mounting container, check configurations ")



import os
import subprocess
import io
import uuid
from azure.storage.blob import BlobServiceClient
import sys
from azure.storage.filedatalake import DataLakeServiceClient
from azure.core._match_conditions import MatchConditions
from azure.storage.filedatalake._models import ContentSettings
from azure.storage.fileshare import ShareFileClient
    
# This library takes these docs:
# https://docs.microsoft.com/en-us/azure/storage/blobs/data-lake-storage-directory-file-acl-python
# and puts them into library form.
# There is no new information here of any kind, it is just a clean up of the above docs.
# The above docs show some examples of how to use the datalake client, I then went to the
# full rest api and implemented more pieces of that for greater functionality in python.
# There are no new features here.
# full rest api: https://docs.microsoft.com/en-us/python/api/azure-storage-file-datalake/azure.storage.filedatalake?view=azure-python

# This works on Continuous Deployment generically
# other solutions the author found only work on databricks notebooks

class AzureDataLakeClient:
    def __init__(self, storage_account_name, file_system_name, storage_account_key='', client_id='', client_secret='', tenant_id=''):
        self.storage_account_name = storage_account_name
        if storage_account_key != '':
            
            self.service_client = self.initialize_storage_account(
                storage_account_name,
                storage_account_key
            )
        elif client_id != '':
            self.service_client = self.initialize_storage_account_ad(
                storage_account_name,
                client_id,
                client_secret,
                tenant_id
            )
        self.file_system_client = self.get_or_create_file_system(
            file_system_name
        )
        
    def initialize_storage_account(self, storage_account_name, storage_account_key):
        try:  
            return DataLakeServiceClient(
                account_url=f"https://{storage_account_name}.dfs.core.windows.net",
                credential=storage_account_key
            )

        except Exception as e:
            raise e

    def initialize_storage_account_ad(self, storage_account_name, client_id, client_secret, tenant_id):
        try:  
            credential = ClientSecretCredential(tenant_id, client_id, client_secret)
            return DataLakeServiceClient(
                account_url=f"https://{storage_account_name}.dfs.core.windows.net",
                credential=credential
            )

        except Exception as e:
            raise e

    def get_or_create_file_system(self, file_system_name):
        try:
            try:
                return self.service_client.create_file_system(
                    file_system=file_system_name
                )
            except:
                return self.service_client.get_file_system_client(
                    file_system=file_system_name
                )
        except Exception as e:
            raise e

    def create_directory(self, directory_name):
        try:
            self.file_system_client.create_directory(
                directory_name
            )
        except Exception as e:
            raise e

    def rename_directory(self, dir_name, new_dir_name):
        try:
            dir_client = self.file_system_client.get_directory_client(
                dir_name
            )
            new_dir_path = dir_client.file_system_name + "/" + new_dir_name
            dir_client.rename_directory(new_name=new_dir_path)
        except Exception as e:
            raise e
        
    def delete_directory(self, dir_name):
        try:
            dir_client = self.file_system_client.get_directory_client(dir_name)
            dir_client.delete_directory()
        except Exception as e:
            raise e

    def upload_file(self, local_file_path, remote_file_path, mode='r'):
        try:
            file_client = self.get_file_client(remote_file_path)
            with open(local_file_path, mode) as local_file:
                contents = local_file.read()
            file_client.append_data(
                data=contents,
                offset=0,
                length=len(contents)
            )
            file_client.flush_data(len(contents))
        except Exception as e:
            raise e

    def get_file_client(self, remote_file_path):
        try:
            remote_file = remote_file_path.split("/")[-1]
            remote_dir = "/".join(remote_file_path.split("/")[:-1])
            dir_client = self.file_system_client.get_directory_client(remote_dir)
            return dir_client.create_file(remote_file)
        except Exception as e:
            raise e

    def download_file(self, remote_file_path, local_file_path):
        try:
            file_client = self.get_file_client(remote_file_path)
            download = file_client.download_file()
            downloaded_bytes = download.readall()
            with open(local_file_path, 'wb') as local_file:
                local_file.write(downloaded_bytes)
            
        except Exception as e:
            raise e

    def ls(self, remote_dir):
        try:
            paths = self.file_system_client.get_paths(
                path=remote_dir
            )
            for path in paths:
                print(path.name)
        except Exception as e:
            raise e

    def folder_exists(self, remote_file_path):
        path = "/".join(remote_file_path.split("/")[:-1])
        try:
            dir_client = self.file_system_client.get_directory_client(
                path
            )
            return dir_client.exists()
        except Exception as e:
            raise e

    def makedirs(self, remote_file_path):
        num_nested_folders = len(remote_file_path.split("/")[:-1])
        for index in range(num_nested_folders):
            if not self.folder_exists(remote_file_path, index):
                path = "/".join(remote_file_path.split("/")[:index+1])
                self.create_directory(path)
        
    def copy_file(self, local_file_path, remote_file_path):
        folder_path = "/".join(remote_file_path.split("/")[:-1])
        if not self.folder_exists(folder_path):
            self.makedirs(folder_path)
        self.upload_file(
            local_file_path, remote_file_path
        )

    # run before orchestrator
    def recursive_copy(self, root, run_type, version_id, repo_name):
        """
        options for run_type: testing, production, staging
        """
        files_to_copy = []
        for rel_root, directory, files in os.walk(root):
            for File in files:
                files_to_copy.append(
                    os.path.join(rel_root, File)
                )
        for file_path in files_to_copy:
            self.copy_file(
                file_path,
                f"{repo_name}/{run_type}/{version_id}/{file_path}"
            )

def generate_meta_data(
        version_id, run_type, job_id,
        run_id, repo_name, commit_hash
):
    local_file = f"metadata_{version_id}.txt"
    with open(local_file, "w") as f:
        f.write(f"""
        Metadata:
        * job id {job_id}
        * run id {run_id}
        * repo name {repo_name}
        * commit hash {commit_hash}
        * run type {run_type}
        """)
    return local_file

    
def copy_to_blob_storage(connection_string, file_name, container_name):
    blob_service_client = BlobServiceClient.from_connection_string(connection_string)

    blob_client = blob_service_client.get_blob_client(
        container=container_name, blob="dags/"+file_name)

    with open(file_name, "rb") as data:
        blob_client.upload_blob(data, overwrite=True)


def copy_to_file_share(connection_string, file_name, share_name):
    file_client = ShareFileClient.from_connection_string(
        conn_str=connection_string, share_name=share_name, file_path="dags/"+file_name
    )

    with open(file_name, "rb") as source_file:
        file_client.upload_file(source_file)



from typing import List, Dict
import requests


def create_job(
    name: str,
    tasks: List[Dict],
    job_clusters: list,
    email_notifications: dict,
    timeout_seconds: int,
    schedule: dict,
    max_concurrent_runs: int,
    job_format: str,
    access_control_list: List[Dict],
    host: str,
    headers: dict,
) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsCreate

    Create a new job.  This is the main work horse of the library.

    Parameters
    ----------
    * name : str - An optional name for the job.

    Default: "Untitled"

    * tags : dict - A map of tags associated with the job.
    These are forwarded to the cluster as cluster tags for
    jobs clusters, and are subject to the same limitations as
    cluster tags.  A maximum of 25 tags can be added to the job.

    Default: empty dict = {}

    * tasks : list of dicts - a list of task
    specfications to be executed by this job.

    Default: list with empty dict = [{}]

    Note: You can specify a max of 100 items.

    Example task:

    {
    "task_key": "Sessionize",
    "description": "Extracts session data from events",
    "depends_on": [ ],
    "existing_cluster_id": "0923-164208-meows279",
    "spark_jar_task": {
    "main_class_name": "com.databricks.Sessionize",

    "parameters": [
      "--data", "dbfs:/path/to/data.json"
    ]

    },
     "libraries": [
      {
       "jar": "dbfs:/mnt/databricks/Sessionize.jar"
      }
     ],
       "timeout_seconds": 86400,
       "max_retries": 3,
       "min_retry_interval_millis": 2000,
       "retry_on_timeout": false
     },

    * job_clusters : list of dicts - A list of job
    cluster specifications that can be shared and
    reused by tasks of this job. Libraries cannot be
    declared in a shared job cluster.  You must declare
    dependent libraries in task settings.

    Default: list with empty dict = [{}]

    Note You can specify a max of 100 items.

    * email_notifications : dict -

    Keys:
      * on_start - who to email on start of job
      * on_success - who to email on success of job
      * on_failure - who to email on failure of job
      * no_alert_for_skipped_runs - boolean

    * timeout_seconds : int - An optional timeout
    applied to each run of this job.  The default
    behavior is to have no timeout.

    * schedule : dict

    Keys:
    "quartz_cron_expression": follows cron expression,
    like found here:
    https://www.freeformatter.com/cron-expression-generator-quartz.html

    "timezone_id": follows java time zones like found here:

    https://docs.trifacta.com/display/DP/Supported+Time+Zone+Values

    "pause_status": "PAUSED" or "UNPAUSED"

    max_concurrent_runs : int - An optional maximum allowed number
    of concurrent runs of the job.

    Set this value if you want to be able to execute multiple runs of
    the same job concurrently.  This is useful for example if you
    trigger your job on a frequent schedule and want to allow consecutive
    runs to overlap with each other, or if you want to trigger multiple
    runs which differ by their input parameters.

    This setting affects only new runs.  For example, suppose the
    job's concurrency is 4 and there are 4 concurrent active runs.
    Then setting the concurrency to 3 won't kill any of the active runs.
    However, from then on, new runs are skipped unless there are fewer than
    3 active runs.

    This value cannot exceed 1000.  Setting this value to 0 causes all new
    runs to be skipped.

    Default: 1.

    * job_format : str - Used to tell what the format of the job is.
    This field is ignored in Create/Update/Reset calls.

    Values: "SINGLE_TASK" or "MULTI_TASK"

    Default: "MULTI_TASK"

    * access_control_list : list of dicts -

    Whoever creates the job should always set
    {
     "user_name": [YOUR EMAIL GOES HERE],
     "permission_level": "IS_OWNER"
    }

    You can also set secondary users to "permission_level": "CAN_MANAGE"

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed
    """
    parameters = {
        "name": name,
        "tasks": tasks,
        "job_clusters": job_clusters,
        "email_notifications": email_notifications,
        "timeout_seconds": str(timeout_seconds),
        "schedule": schedule,
        "max_concurrent_runs": max_concurrent_runs,
        "format": job_format,
        "access_control_list": access_control_list,
    }

    response = requests.post(
        f"{host}/api/2.1/jobs/create", headers=headers, json=parameters
    )
    return response


def get_jobs(
    limit: int, offset: int, expand_tasks: bool, host: str, api: str, headers: dict
) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsList

    Retrieves a list of jobs.

    Parameters
    ----------

    * limit : int - The number of jobs to return.
    0 < limit <= 25.  Default is 20.

    * offset : int - The offset of the first job
    to return, relative to the most recently created
    job.

    Example:

    Offset = 0, returns the most recently created job.

    * expand_tasks : bool -
    Whether to include task and cluster details in the
    response.

    * host : str - the host IP Address

    * api : str - the api version to use.

    Note: 2.1 is suggested

    * headers: dict - the headers come from the config
    and should not be changed
    """
    parameters = {
        "limit": str(limit),
        "offset": str(offset),
        "expand_tasks": str(expand_tasks).lower(),
    }
    response = requests.get(f"{host}{api}/jobs/list", headers=headers, json=parameters)
    return response


def get_job(job_id: int, host: str, headers: dict) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsGet

    Retrieves the details for a single job.

    Parameters
    ----------

    * job_id : int - The canonical identifier of the job to
    retrieve information about.

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed
    """
    parameters = {"job_id": str(job_id)}

    response = requests.get(
        f"{host}/api/2.1/jobs/get", headers=headers, json=parameters
    )
    return response


def update_job_settings(
    job_id: int, new_settings: dict, host: str, headers: dict
) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsUpdate

    Add, update, or remove specific settings of an existing job.
    Use the Reset endpoint to overwrite all job settings.

    Parameters
    ----------

    * job_id : int - The canonical identifier of the job to
    retrieve information about.

    * new_settings: dict - A list of new settings.

    Settings that can be updated:
        * name
        * tasks
        * job_clusters
        * email_notifications
        * timeout_seconds
        * schedule
        * max_concurrent_runs
        * job_format
        * access_control_list

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed
    """
    job_parameters = get_job(job_id, host, api)
    for key in job_parameters:
        if key in new_settings:
            job_parameters[key] = new_settings[key]

    parameters = {"job_id": str(job_id), "new_settings": job_parameters}
    response = requests.post(
        f"{host}/api/2.1/jobs/update", headers=headers, json=parameters
    )
    return response


def reset_job_settings(
    job_id: int,
    name: str,
    tasks: List[Dict],
    job_clusters: list,
    email_notifications: dict,
    timeout_seconds: int,
    schedule: dict,
    max_concurrent_runs: int,
    job_format: str,
    access_control_list: List[Dict],
    host: str,
    headers: dict,
) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsReset

    Overwrites all the settings for a specific job. Use the Update endpoint to update job settings partially.

    Parameters
    ----------

    * job_id : int - The canonical identifier of the job to
    retrieve information about.


    * name : str - An optional name for the job.

    Default: "Untitled"

    * tags : dict - A map of tags associated with the job.
    These are forwarded to the cluster as cluster tags for
    jobs clusters, and are subject to the same limitations as
    cluster tags.  A maximum of 25 tags can be added to the job.

    Default: empty dict = {}

    * tasks : list of dicts - a list of task
    specfications to be executed by this job.

    Default: list with empty dict = [{}]

    Note: You can specify a max of 100 items.

    Example task:

    {
    "task_key": "Sessionize",
    "description": "Extracts session data from events",
    "depends_on": [ ],
    "existing_cluster_id": "0923-164208-meows279",
    "spark_jar_task": {
    "main_class_name": "com.databricks.Sessionize",

    "parameters": [
      "--data", "dbfs:/path/to/data.json"
    ]

    },
     "libraries": [
      {
       "jar": "dbfs:/mnt/databricks/Sessionize.jar"
      }
     ],
       "timeout_seconds": 86400,
       "max_retries": 3,
       "min_retry_interval_millis": 2000,
       "retry_on_timeout": false
     },

    * job_clusters : list of dicts - A list of job
    cluster specifications that can be shared and
    reused by tasks of this job. Libraries cannot be
    declared in a shared job cluster.  You must declare
    dependent libraries in task settings.

    Default: list with empty dict = [{}]

    Note You can specify a max of 100 items.

    * email_notifications : dict -

    Keys:
      * on_start - who to email on start of job
      * on_success - who to email on success of job
      * on_failure - who to email on failure of job
      * no_alert_for_skipped_runs - boolean

    * timeout_seconds : int - An optional timeout
    applied to each run of this job.  The default
    behavior is to have no timeout.

    * schedule : dict

    Keys:
    "quartz_cron_expression": follows cron expression,
    like found here:
    https://www.freeformatter.com/cron-expression-generator-quartz.html

    "timezone_id": follows java time zones like found here:

    https://docs.trifacta.com/display/DP/Supported+Time+Zone+Values

    "pause_status": "PAUSED" or "UNPAUSED"

    max_concurrent_runs : int - An optional maximum allowed number
    of concurrent runs of the job.

    Set this value if you want to be able to execute multiple runs of
    the same job concurrently.  This is useful for example if you
    trigger your job on a frequent schedule and want to allow consecutive
    runs to overlap with each other, or if you want to trigger multiple
    runs which differ by their input parameters.

    This setting affects only new runs.  For example, suppose the
    job's concurrency is 4 and there are 4 concurrent active runs.
    Then setting the concurrency to 3 won't kill any of the active runs.
    However, from then on, new runs are skipped unless there are fewer than
    3 active runs.

    This value cannot exceed 1000.  Setting this value to 0 causes all new
    runs to be skipped.

    Default: 1.

    * job_format : str - Used to tell what the format of the job is.
    This field is ignored in Create/Update/Reset calls.

    Values: "SINGLE_TASK" or "MULTI_TASK"

    Default: "MULTI_TASK"

    * access_control_list : list of dicts -

    Whoever creates the job should always set
    {
     "user_name": [YOUR EMAIL GOES HERE],
     "permission_level": "IS_OWNER"
    }

    You can also set secondary users to "permission_level": "CAN_MANAGE"

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed
    """

    parameters = {
        "job_id": str(job_id),
        "new_settings": {
            "name": name,
            "tasks": tasks,
            "job_clusters": job_clusters,
            "email_notifications": email_notifications,
            "timeout_seconds": str(timeout_seconds),
            "schedule": schedule,
            "max_concurrent_runs": max_concurrent_runs,
            "format": job_format,
            "access_control_list": access_control_list,
        },
    }

    response = requests.post(
        f"{host}/api/2.1/jobs/update", headers=headers, json=parameters
    )
    return response


def delete_job(job_id: int, host: str, headers: dict) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsDelete

    Parameters
    ----------

    * job_id : int - The canonical identifier of the job to
    retrieve information about.

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed

    """
    parameters = {"job_id": str(job_id)}

    response = requests.post(
        f"{host}/api/2.1/jobs/delete", headers=headers, json=parameters
    )
    return response


def execute_job(
    job_id: int,
    host: str,
    headers: dict,
    idempotency_token: str = None,
    jar_params: List = None,
    notebook_params: List[Dict] = None,
    python_params: List = None,
    spark_submit_params: List = None,
    pipeline_params: dict = {"full_refresh": False},
) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsRunNow

    Run a job and return the run_id of the triggered run.

    Parameters
    ----------

    * job_id : int - The canonical identifier of the job to
    retrieve information about.

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed

    * pipeline_params : dict -

    A dictionary with one key-value pair:

       full_refresh : bool - If true, triggers a full refresh on the delta live table.

    * idempotency_token : str - [Optional]

    An optional token to guarantee the idempotency of job run requests.
    If a run with the provided token already exists,
    the request does not create a new run but returns the
    ID of the existing run instead.
    If a run with the provided token is deleted, an error is returned.

    If you specify the idempotency token,
    upon failure you can retry until the request succeeds.
    Databricks guarantees that exactly one run is launched with that idempotency token.

    This token must have at most 64 characters.


    The following parameters are optional, but at most
    one of the following can be passed:

    * jar_params : list - A list of parameters for jobs with Spark JAR tasks,
    for example "jar_params": ["john doe", "35"]. The parameters are used to invoke
    the main function of the main class specified in the Spark JAR task.
    If not specified upon run-now, it defaults to an empty list.
    jar_params cannot be specified in conjunction with notebook_params.
    The JSON representation of this field
    (for example {"jar_params":["john doe","35"]}) cannot exceed 10,000 bytes.

    * notebook_params : list[dict] -
    A map from keys to values for jobs with notebook task,
    for example "notebook_params": {"name": "john doe", "age": "35"}.
    The map is passed to the notebook and is
    accessible through the dbutils.widgets.get function.
    If not specified upon run-now, the triggered run uses the job’s base parameters.
    notebook_params cannot be specified in conjunction with jar_params.
    Use Task parameter variables to set parameters containing information about job runs.
    The JSON representation of this field (for example {"notebook_params":{"name":"john doe","age":"35"}})
    cannot exceed 10,000 bytes.

    * python_params : list -

    A list of parameters for jobs with Python tasks,
    for example "python_params": ["john doe", "35"].
    The parameters are passed to Python file as command-line parameters.
    If specified upon run-now, it would overwrite the parameters specified in job setting.
    The JSON representation of this field
    (for example {"python_params":["john doe","35"]}) cannot exceed 10,000 bytes.

    Use Task parameter variables to set parameters containing information about job runs.

    Important

    These parameters accept only Latin characters (ASCII character set).
    Using non-ASCII characters returns an error.
    Examples of invalid, non-ASCII characters are Chinese, Japanese kanjis, and emojis.

    * spark_submit_params : list -
    A list of parameters for jobs with spark submit task,
    for example "spark_submit_params":
    ["--class", "org.apache.spark.examples.SparkPi"].
    The parameters are passed to spark-submit script as command-line parameters.
    If specified upon run-now, it would overwrite the parameters specified in job setting.
    The JSON representation of this field
    (for example {"python_params":["john doe","35"]}) cannot exceed 10,000 bytes.

    Use Task parameter variables to set parameters containing information about job runs.

    Important

    These parameters accept only Latin characters (ASCII character set).
    Using non-ASCII characters returns an error. Examples of invalid,
    non-ASCII characters are Chinese, Japanese kanjis, and emojis.
    """
    parameters = {
        "job_id": str(job_id),
    }
    if idempotency_token:
        parameters["idempotency_token"] = idempotency_token
    if jar_params:
        parameters["jar_params"] = jar_parms
    if notebook_params:
        parameters["notebook_params"] = notebook_params
    if python_params:
        parameters["python_params"] = python_params
    if spark_submit_params:
        parameters["spark_submit_params"] = spark_submit_params

    response = requests.post(
        f"{host}/api/2.1/jobs/run-now", headers=headers, json=parameters
    )
    return response


def execute_standalone_job(
    tasks: List[Dict],
    job_clusters: List[Dict],
    run_name: str,
    timeout_seconds: int,
    idempotency_token: str,
    access_control_list: List[Dict],
    host: str,
    headers: dict,
) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsRunsSubmit

    Submit a one-time run.
    This endpoint allows you to submit a workload directly without creating a job.
    Runs submitted using this endpoint don’t display in the UI.
    Use the jobs/runs/get API to check the run state after the job is submitted.

    Parameters
    ----------
    * tasks : list of dicts - a list of task
    specfications to be executed by this job.

    Default: list with empty dict = [{}]

    Note: You can specify a max of 100 items.

    Example task:

    {
    "task_key": "Sessionize",
    "description": "Extracts session data from events",
    "depends_on": [ ],
    "existing_cluster_id": "0923-164208-meows279",
    "spark_jar_task": {
    "main_class_name": "com.databricks.Sessionize",

    "parameters": [
      "--data", "dbfs:/path/to/data.json"
    ]

    },
     "libraries": [
      {
       "jar": "dbfs:/mnt/databricks/Sessionize.jar"
      }
     ],
       "timeout_seconds": 86400,
       "max_retries": 3,
       "min_retry_interval_millis": 2000,
       "retry_on_timeout": false
     },

    * job_clusters : list of dicts - A list of job
    cluster specifications that can be shared and
    reused by tasks of this job. Libraries cannot be
    declared in a shared job cluster.  You must declare
    dependent libraries in task settings.

    Default: list with empty dict = [{}]

    Note You can specify a max of 100 items.

    * email_notifications : dict -

    Keys:
      * on_start - who to email on start of job
      * on_success - who to email on success of job
      * on_failure - who to email on failure of job
      * no_alert_for_skipped_runs - boolean

    * run_name : str -

    An optional name for the run. The default value is Untitled

    * timeout_seconds : int - An optional timeout
    applied to each run of this job.  The default
    behavior is to have no timeout.

    * idempotency_token : str - [Optional]

    An optional token to guarantee the idempotency of job run requests.
    If a run with the provided token already exists,
    the request does not create a new run but returns the
    ID of the existing run instead.
    If a run with the provided token is deleted, an error is returned.

    If you specify the idempotency token,
    upon failure you can retry until the request succeeds.
    Databricks guarantees that exactly one run is launched with that idempotency token.

    This token must have at most 64 characters.

    * access_control_list : list of dicts -

    Whoever creates the job should always set
    {
     "user_name": [YOUR EMAIL GOES HERE],
     "permission_level": "IS_OWNER"
    }

    You can also set secondary users to "permission_level": "CAN_MANAGE"

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed
    """
    parameters = {}
    parameters["tasks"] = tasks
    parameters["run_name"] = run_name,
    parameters["timeout_seconds"] = str(timeout_seconds),
    parameters["access_control_list"] = access_control_list
    
    if job_clusters:
        parameters["job_clusters"] = job_clusters
    if idempotency_token:
        parameters["idempotency_token"] = idempotency_token
        
    response = requests.post(
        f"{host}/api/2.1/jobs/runs/submit", headers=headers, json=parameters
    )
    return response


def get_runs_for_job(
    active_only: bool,
    completed_only: bool,
    offset: int,
    limit: int,
    job_id: int,
    run_type: str,
    expand_tasks: bool,
    host: str,
    headers: dict,
) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsRunsList

    List runs in descending order by start time.

    Parameters
    -----------

    * active_only : bool -
    If active_only is true only active runs are included.

    * completed_only : bool -
    If completed_only is true, only completed runs
    are included in the results.

    * offset : int - The offset of the first job
    to return, relative to the most recently created
    job.

    * limit : int - The number of jobs to return.
    0 < limit <= 25.  Default is 20.

    * job_id : int - The canonical identifier of the job to
    retrieve information about.

    * run_type : str -

    The type of runs to return. For a description of run types.
    Options:
    * JOB_RUN
    * WORKFLOW_RUN
    * SUBMIT_RUN

    * expand_tasks : bool -
    Whether to include task and cluster details in the response.

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed
    """
    parameters = {
        "job_id": str(job_id),
        "active_only": str(active_only).lower(),
        "completed_only": str(completed_only).lower(),
        "offset": str(offset),
        "lmit": str(limit),
        "run_type": run_type,
        "expand_tasks": str(expand_tasks).lower(),
    }

    response = requests.post(
        f"{host}/api/2.1/jobs/runs/list", headers=headers, json=parameters
    )

    return response


def get_runs_for_joblist(
    active_only: bool,
    completed_only: bool,
    offset: int,
    limit: int,
    run_type: str,
    expand_tasks: bool,
    start_time_from: int,
    start_time_to: int,
    host: str,
    headers: dict,
) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsRunsList

    List runs in descending order by start time.

    Parameters
    ----------

    * active_only : bool -
    If active_only is true only active runs are included.

    * completed_only : bool -
    If completed_only is true, only completed runs
    are included in the results.

    * expand_tasks : bool -
    Whether to include task and cluster details in the
    response.

    * limit : int - The number of jobs to return.
    0 < limit <= 25.  Default is 20.

    * run_type : str -

    The type of runs to return. For a description of run types.
    Options:
    * JOB_RUN
    * WORKFLOW_RUN
    * SUBMIT_RUN

    * offset : int - The offset of the first job
    to return, relative to the most recently created
    job.

    * start_time_from : int - show runs that started at or after
    this value.  The value must be a UTC timestamp in milliseconds.

    * start_time_to : int - show runs that started at or before
    this value.  The value must be a UTC timestamp in milliseconds.

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed

    """
    parameters = {
        "active_only": str(active_only).lower(),
        "completed_only": str(completed_only).lower(),
        "offset": str(offset),
        "lmit": str(limit),
        "run_type": run_type,
        "expand_tasks": str(expand_tasks).lower(),
        "start_time_from": start_time_from,
        "start_time_to": start_time_to,
    }

    response = requests.post(
        f"{host}/api/2.1/jobs/runs/list", headers=headers, json=parameters
    )
    return response


def get_run(run_id: int, host: str, headers: dict) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsRunsGet

    Retrieve the metadata for a run.

    Parameters
    -----------
    * run_id : int -
    The canonical identifier of the run for which to
    retrieve the metadata.

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed
    """

    parameters = {"run_id": str(run_id)}

    response = requests.get(
        f"{host}/api/2.1/jobs/runs/get", headers=headers, json=parameters
    )
    return response


def copy_code_to_notebook(
    run_id: int, host: str, headers: dict, views_to_export: str = "CODE"
) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsRunsExport

    Export and retrieve the job run task.

    Parameters
    ----------

    * run_id : int -
    The canonical identifier of the run for which to
    retrieve the metadata.

    * views_to_export : str -

    An enumerable with values:
    * CODE
    * DASHBOARDS
    * ALL

    Default is CODE.

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed

    """
    parameters = {"run_id": str(run_id), "views_to_export": views_to_export}
    response = requests.post(
        f"{host}/api/2.1/jobs/runs/export", headers=headers, json=parameters
    )
    return response


def cancel_run(run_id: str, host: str, headers: dict) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsRunsCancel

    Cancels a job run. The run is canceled asynchronously,
    so it may still be running when this request completes.

    Parameters
    ----------
    * run_id : int -
    The canonical identifier of the run for which to
    retrieve the metadata.

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed

    """
    parameters = {"run_id": str(run_id)}
    response = requests.post(
        f"{host}/api/2.1/jobs/runs/cancel", headers=headers, json=parameters
    )
    return response


def get_output(run_id: str, host: str, headers: dict) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsRunsGetOutput

    Retrieve the output and metadata of a run.
    When a notebook task returns a value through the dbutils.notebook.exit() call,
    you can use this endpoint to retrieve that value.
    Databricks restricts this API to return the first 5 MB of the output.
    To return a larger result, you can store job results in a cloud storage service.
    This endpoint validates that the run_id parameter is valid and returns an HTTP
    status code 400 if the run_id parameter is invalid. Runs are automatically
    removed after 60 days. If you to want to reference them beyond 60 days,
    you must save old run results before they expire.
    To export using the UI, see Export job run results.
    To export using the Jobs API, see Runs export.

    Parameters
    ----------
    * run_id : int -
    The canonical identifier of the run for which to
    retrieve the metadata.

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed

    """
    parameters = {"run_id": str(run_id)}
    response = requests.post(
        f"{host}/api/2.1/jobs/runs/get-output", headers=headers, json=parameters
    )
    return response


def delete_run(run_id: str, host: str, headers: dict) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsRunsDelete

    Deletes a non-active run. Returns an error if the run is active.

    Parameters
    ----------
    * run_id : int -
    The canonical identifier of the run for which to
    retrieve the metadata.

    * host : str - the host IP Address

    * headers: dict - the headers come from the config
    and should not be changed

    """
    parameters = {"run_id": str(run_id)}
    response = requests.post(
        f"{host}/api/2.1/jobs/runs/delete", headers=headers, json=parameters
    )
    return response


######################################################################################
# Databricks Repos API - https://docs.databricks.com/dev-tools/api/latest/repos.html #
######################################################################################
def get_repos(
        host: str,
        path_prefix: str,
        next_page_token: str,
        headers:dict
    ) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/repos.html#operation/get-repos

    Returns repos that the calling user has Manage permissions on. Results are paginated with each page containing twenty repos.

    Parameters
    ----------
    * host: str - the host IP Address

    * path_prefix : str - Filters repos that have paths starting with the given path prefix.

    * next_page_token : str - Token used to get the next page of results.
    If not specified, returns the first page of results as well as a next page token if there are more results.

    * headers: dict - the headers come from the config
    and should not be changed
    """
    parameters = {
        "path_prefix": str(path_prefix),
        "next_page_token": str(next_page_token)
    }
    response = requests.get(
        f'{host}/api/2.1/repos',
        headers=headers,
        json=parameters
    )
    return response

def create_repo(
        host: str,
        url: str,
        provider: str,
        path: str,
        headers: dict
    ) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/repos.html#operation/create-repo

    Creates a repo in the workspace and links it to the remote Git repo specified.
    Note that repos created programmatically must be linked to a remote Git repo, unlike repos created in the browser.

    Parameters
    ----------

    * host: str - the host IP Address

    * url: str - URL of the Git repository to be linked.

    * provider: str - Git provider. Case insensitive. Available providers are:
        - gitHub
        - bitbucketCloud
        - gitLab
        - azureDevOpsServices
        - gitHubEnterprise
        - bitbucketServer
        - gitLabEnterpriseEdition
        - awsCodeCommit

    * path: str - Desired path for the repo in the workspace. Must be in the format /Repos/{folder}/{repo-name}.

    * header: dict - the headers come from the config and should not be changed
    """
    parameters = {
        "url": str(url),
        "provider": str(provider),
        "path": str(path)
    }
    response = requests.post(
        f'{host}/api/2.1/repos',
        headers=headers,
        json=parameters
    )
    return response

def update_repo(
        host: str,
        repo_id: str,
        headers: dict,
        branch: str = None,
        tag: str = None,
    ) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/repos.html#operation/update-repo

    Updates the repo to a different branch or tag, or updates the repo to the latest commit on the same branch.


    Parameters
    ----------

    * host: str - the host IP Address

    Requires one of the following to update the repo:
        * branch: str - Branch that the local version of the repo is checked out to.
        * tag: str - Tag that the local version of the repo is checked out to.
                Updating the repo to a tag puts the repo in a detached HEAD state.
                Before committing new changes, you must update the repo to a branch instead of the detached HEAD.

    * repo_id: str - The ID for the corresponding repo to access.

    * headers: dict - the headers come from the config and should not be changed
    """
    parameters = {}
    if branch and tag:
        raise Exception('Both branch and tag parameters cannot be passed together.')
    if branch:
        parameters["branch"] = str(branch)
    elif tag:
        parameters["tag"] = str(tag)
    response = requests.patch(
        f'{host}/api/2.1/repos/{repo_id}',
        headers=headers,
        json=parameters
    )
    return response

def delete_repo(
        host: str,
        repo_id: str,
        headers: dict
    ) -> requests.Response:
    """
    Original Docs: https://docs.databricks.com/dev-tools/api/latest/repos.html#operation/delete-repo

    Deletes the specified repo.

    Parameters
    ----------

    * host: str - the host IP Address

    * repo_id: str - The ID for the corresponding repo to access.

    * headers: dict - the headers come from the config and should not be changed
    """
    parameters = {
        "repo_id": str(repo_id)
    }
    response = requests.post(
        f'{host}/api/2.1/repos/{repo_id}',
        headers=headers,
        json=parameters
    )
    return response



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="data_pipeline_tooling",
    version="0.12",
    description="A library for databricks jobs api",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Eric Schles",
    author_email="eschles3@jh.edu",
    url="https://github.com/JH-PMAP/data_pipeline_tooling",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9"
    ],
    packages=["data_pipeline_tooling"],
    include_package_data=True,
    install_requires=[
        "requests", "fire", "azure-storage-blob", "azure-storage-file-datalake", "azure-core",
        "azure-storage-file-share"
    ],
    console_scripts=["scripts/orchestrator_cli"],
    scripts=["scripts/orchestrator_cli"],
)
